]> git.sesse.net Git - stockfish/blobdiff - src/tt.h
Sync with master
[stockfish] / src / tt.h
index eeb18e32acff0510656537dad8cdac29b4a8f1f6..32b4036e39dea085974410867b7212facfc6c625 100644 (file)
--- a/src/tt.h
+++ b/src/tt.h
@@ -1,7 +1,7 @@
 /*
   Stockfish, a UCI chess playing engine derived from Glaurung 2.1
   Copyright (C) 2004-2008 Tord Romstad (Glaurung author)
-  Copyright (C) 2008-2014 Marco Costalba, Joona Kiiski, Tord Romstad
+  Copyright (C) 2008-2015 Marco Costalba, Joona Kiiski, Tord Romstad
 
   Stockfish is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
 #include "misc.h"
 #include "types.h"
 
/// The TTEntry is the 14 bytes transposition table entry, defined as below:
 ///
-/// key        32 bit
+/// key        16 bit
 /// move       16 bit
-/// bound type  8 bit
-/// generation  8 bit
 /// value      16 bit
-/// depth      16 bit
 /// eval value 16 bit
+/// generation  6 bit
+/// bound type  2 bit
+/// depth       8 bit
 
 struct TTEntry {
 
-  Move  move()  const      { return (Move )move16; }
-  Bound bound() const      { return (Bound)bound8; }
-  Value value() const      { return (Value)value16; }
-  Depth depth() const      { return (Depth)depth16; }
-  Value eval_value() const { return (Value)evalValue; }
+  Move  move()  const { return (Move )move16; }
+  Value value() const { return (Value)value16; }
+  Value eval()  const { return (Value)eval16; }
+  Depth depth() const { return (Depth)depth8; }
+  Bound bound() const { return (Bound)(genBound8 & 0x3); }
 
-private:
-  friend class TranspositionTable;
+  void save(Key k, Value v, Bound b, Depth d, Move m, Value ev, uint8_t g) {
 
-  void save(uint32_t k, Value v, Bound b, Depth d, Move m, uint8_t g, Value ev) {
+    if (m || (k >> 48) != key16) // Preserve any existing move for the same position
+        move16 = (uint16_t)m;
 
-    key32       = (uint32_t)k;
-    move16      = (uint16_t)m;
-    bound8      = (uint8_t)b;
-    generation8 = (uint8_t)g;
-    value16     = (int16_t)v;
-    depth16     = (int16_t)d;
-    evalValue   = (int16_t)ev;
+    key16     = (uint16_t)(k >> 48);
+    value16   = (int16_t)v;
+    eval16    = (int16_t)ev;
+    genBound8 = (uint8_t)(g | b);
+    depth8    = (int8_t)d;
   }
 
-  uint32_t key32;
+private:
+  friend class TranspositionTable;
+
+  uint16_t key16;
   uint16_t move16;
-  uint8_t bound8, generation8;
-  int16_t value16, depth16, evalValue;
+  int16_t  value16;
+  int16_t  eval16;
+  uint8_t  genBound8;
+  int8_t   depth8;
 };
 
 
@@ -70,35 +73,37 @@ private:
 
 class TranspositionTable {
 
-  static const unsigned ClusterSize = 4;
+  static const int CacheLineSize = 64;
+  static const int ClusterSize = 3;
+
+  struct Cluster {
+    TTEntry entry[ClusterSize];
+    char padding[2]; // Align to the cache line size
+  };
+
+  static_assert(sizeof(Cluster) == CacheLineSize / 2, "Cluster size incorrect");
 
 public:
  ~TranspositionTable() { free(mem); }
-  void new_search() { ++generation; }
-
-  const TTEntry* probe(const Key key) const;
-  TTEntry* first_entry(const Key key) const;
-  void resize(uint64_t mbSize);
+  void new_search() { generation8 += 4; } // Lower 2 bits are used by Bound
+  uint8_t generation() const { return generation8; }
+  TTEntry* probe(const Key key, bool& found) const;
+  int hashfull() const;
+  void resize(size_t mbSize);
   void clear();
-  void store(const Key key, Value v, Bound type, Depth d, Move m, Value statV);
+
+  // The lowest order bits of the key are used to get the index of the cluster
+  TTEntry* first_entry(const Key key) const {
+    return &table[(size_t)key & (clusterCount - 1)].entry[0];
+  }
 
 private:
-  uint32_t hashMask;
-  TTEntry* table;
+  size_t clusterCount;
+  Cluster* table;
   void* mem;
-  uint8_t generation; // Size must be not bigger than TTEntry::generation8
+  uint8_t generation8; // Size must be not bigger than TTEntry::genBound8
 };
 
 extern TranspositionTable TT;
 
-
-/// TranspositionTable::first_entry() returns a pointer to the first entry of
-/// a cluster given a position. The lowest order bits of the key are used to
-/// get the index of the cluster.
-
-inline TTEntry* TranspositionTable::first_entry(const Key key) const {
-
-  return table + ((uint32_t)key & hashMask);
-}
-
 #endif // #ifndef TT_H_INCLUDED
@@ -1,7 +1,7 @@
 #include "types.h"
 
-/// The TTEntry is the 14 bytes transposition table entry, defined as below:
+/// TTEntry struct is the 10 bytes transposition table entry, defined as below:
 ///